//
// Glass Display
//
// Copyright (C) 2016 - 2017 Assured Information Security, Inc. All rights reserved.
//
#include "linux/input.h"
#include <xt_input_global.h>

const char *shortcut_strings[221] =
    {
        "reserved",
        "esc",
        "1",
        "2",
        "3",
        "4",
        "5",
        "6",
        "7",
        "8",
        "9",
        "0",
        "-",
        "=",
        "backspace",
        "tab",
        "q",
        "w",
        "e",
        "r",
        "t",
        "y",
        "u",
        "i",
        "o",
        "p",
        "[",
        "]",
        "enter",
        "left-ctrl",
        "a",
        "s",
        "d",
        "f",
        "g",
        "h",
        "j",
        "k",
        "l",
        ";",
        "'",
        "`",
        "left-shift",
        "\\",
        "z",
        "x",
        "c",
        "v",
        "b",
        "n",
        "m",
        ",",
        ".",
        "/",
        "right-shift",
        "kp-asterisk",
        "left-alt",
        "space",
        "caps",
        "f1",
        "f2",
        "f3",
        "f4",
        "f5",
        "f6",
        "f7",
        "f8",
        "f9",
        "f10",
        "numlock",
        "scrolllock",
        "kp-7",
        "kp-8",
        "kp-9",
        "kp-minus",
        "kp-4",
        "kp-5",
        "kp-6",
        "kp-plus",
        "kp-1",
        "kp-2",
        "kp-3",
        "kp-0",
        "kp-dot",
        /* wtf? */ "zenkakuhankaku",
        "102nd",
        "f11",
        "f12",
        "rO",
        "katakana",
        "hiragana",
        "henkan",
        "katakanahiragana",
        "muhenkan",
        "kp-jpcomma",
        "kp-enter",
        "right-ctrl",
        "kp-slash",
        "sysrq",
        "right-alt",
        "linefeed",
        "home",
        "up",
        "pageup",
        "left",
        "right",
        "end",
        "down",
        "pagedown",
        "insert",
        "delete",
        "macro",
        "mute",
        "volumedown",
        "volumeup",
        "power",
        "kp-equal",
        "kp-plusminus",
        "pause",
        "scale",
        "kp-comma",
        "hangeul",
        "hanja",
        "yen",
        "left-meta",
        "right-meta",
        "compose",
        "stop",
        "again",
        "props",
        "undo",
        "front",
        "copy",
        "open",
        "paste",
        "find",
        "cut",
        "help",
        "menu",
        "calc",
        "setup",
        "sleep",
        "wakeup",
        "file",
        "sendfile",
        "deletefile",
        "xfer",
        "prog1",
        "prog2",
        "www",
        "msdos",
        "coffee",
        "screenlock",
        "direction",
        "cyclewindows",
        "mail",
        "bookmarks",
        "computer",
        "back",
        "forward",
        "closecd",
        "ejectcd",
        "ejectclosecd",
        "nextsong",
        "playpause",
        "previoussong",
        "stopcd",
        "record",
        "rewind",
        "phone",
        "iso",
        "config",
        "homepage",
        "refresh",
        "exit",
        "move",
        "edit",
        "scrollup",
        "scrolldown",
        "kpleftparen",
        "kprightparen",
        "new",
        "redo",
        "f13",
        "f14",
        "f15",
        "f16",
        "f17",
        "f18",
        "f19",
        "f20",
        "f21",
        "f22",
        "f23",
        "f24",
        "playcd",
        "pausecd",
        "prog3",
        "prog4",
        "dashboard",
        "suspend",
        "close",
        "play",
        "fastforward",
        "bassboost",
        "print",
        "hp",
        "camera",
        "sound",
        "question",
        "email",
        "chat",
        "search",
        "connect",
        "finance",
        "sport",
        "shop",
        "alterase",
        "cancel",
        "brightnessdown",
        "brightnessup"};

const QMap<QString, int32_t> string_to_keycode{
    {"reserved", KEY_RESERVED},
    {"esc", KEY_ESC},
    {"1", KEY_1},
    {"2", KEY_2},
    {"3", KEY_3},
    {"4", KEY_4},
    {"5", KEY_5},
    {"6", KEY_6},
    {"7", KEY_7},
    {"8", KEY_8},
    {"9", KEY_9},
    {"0", KEY_0},
    {"-", KEY_MINUS},
    {"=", KEY_EQUAL},
    {"backspace", KEY_BACKSPACE},
    {"tab", KEY_TAB},
    {"q", KEY_Q},
    {"w", KEY_W},
    {"e", KEY_E},
    {"r", KEY_R},
    {"t", KEY_T},
    {"y", KEY_Y},
    {"u", KEY_U},
    {"i", KEY_I},
    {"o", KEY_O},
    {"p", KEY_P},
    {"[", KEY_LEFTBRACE},
    {"]", KEY_RIGHTBRACE},
    {"enter", KEY_ENTER},
    {"left-ctrl", KEY_LEFTCTRL},
    {"a", KEY_A},
    {"s", KEY_S},
    {"d", KEY_D},
    {"f", KEY_F},
    {"g", KEY_G},
    {"h", KEY_H},
    {"j", KEY_J},
    {"k", KEY_K},
    {"l", KEY_L},
    {";", KEY_SEMICOLON},
    {"'", KEY_APOSTROPHE},
    {"`", KEY_GRAVE},
    {"left-shift", KEY_LEFTSHIFT},
    {"\\", KEY_BACKSLASH},
    {"z", KEY_Z},
    {"x", KEY_X},
    {"c", KEY_C},
    {"v", KEY_V},
    {"b", KEY_B},
    {"n", KEY_N},
    {"m", KEY_M},
    {",", KEY_COMMA},
    {".", KEY_DOT},
    {"/", KEY_SLASH},
    {"right-shift", KEY_RIGHTSHIFT},
    {"kp-asterisk", KEY_KPASTERISK},
    {"left-alt", KEY_LEFTALT},
    {"space", KEY_SPACE},
    {"caps", KEY_CAPSLOCK},
    {"f1", KEY_F1},
    {"f2", KEY_F2},
    {"f3", KEY_F3},
    {"f4", KEY_F4},
    {"f5", KEY_F5},
    {"f6", KEY_F6},
    {"f7", KEY_F7},
    {"f8", KEY_F8},
    {"f9", KEY_F9},
    {"f10", KEY_F10},
    {"numlock", KEY_NUMLOCK},
    {"scrolllock", KEY_SCROLLLOCK},
    {"kp-7", KEY_KP7},
    {"kp-8", KEY_KP8},
    {"kp-9", KEY_KP9},
    {"kp-minus", KEY_KPMINUS},
    {"kp-4", KEY_KP4},
    {"kp-5", KEY_KP5},
    {"kp-6", KEY_KP6},
    {"kp-plus", KEY_KPPLUS},
    {"kp-1", KEY_KP1},
    {"kp-2", KEY_KP2},
    {"kp-3", KEY_KP3},
    {"kp-0", KEY_KP0},
    {"kp-dot", KEY_KPDOT},
    {/* wtf? */ "zenkakuhankaku", KEY_ZENKAKUHANKAKU},
    {"102nd", KEY_102ND},
    {"f11", KEY_F11},
    {"f12", KEY_F12},
    {"rO", KEY_RO},
    {"katakana", KEY_KATAKANA},
    {"hiragana", KEY_HIRAGANA},
    {"henkan", KEY_HENKAN},
    {"katakanahiragana", KEY_KATAKANAHIRAGANA},
    {"muhenkan", KEY_MUHENKAN},
    {"kp-jpcomma", KEY_KPJPCOMMA},
    {"kp-enter", KEY_KPENTER},
    {"right-ctrl", KEY_RIGHTCTRL},
    {"kp-slash", KEY_KPSLASH},
    {"sysrq", KEY_SYSRQ},
    {"right-alt", KEY_RIGHTALT},
    {"linefeed", KEY_LINEFEED},
    {"home", KEY_HOME},
    {"up", KEY_UP},
    {"pageup", KEY_PAGEUP},
    {"left", KEY_LEFT},
    {"right", KEY_RIGHT},
    {"end", KEY_END},
    {"down", KEY_DOWN},
    {"pagedown", KEY_PAGEDOWN},
    {"insert", KEY_INSERT},
    {"delete", KEY_DELETE},
    {"macro", KEY_MACRO},
    {"mute", KEY_MUTE},
    {"volumedown", KEY_VOLUMEDOWN},
    {"volumeup", KEY_VOLUMEUP},
    {"power", KEY_POWER},
    {"kp-equal", KEY_KPEQUAL},
    {"kp-plusminus", KEY_KPPLUSMINUS},
    {"pause", KEY_PAUSE},
    {"scale", KEY_SCALE},
    {"kp-comma", KEY_KPCOMMA},
    {"hangeul", KEY_HANGEUL},
    {"hanja", KEY_HANJA},
    {"yen", KEY_YEN},
    {"left-meta", KEY_LEFTMETA},
    {"right-meta", KEY_RIGHTMETA},
    {"compose", KEY_COMPOSE},
    {"stop", KEY_STOP},
    {"again", KEY_AGAIN},
    {"props", KEY_PROPS},
    {"undo", KEY_UNDO},
    {"front", KEY_FRONT},
    {"copy", KEY_COPY},
    {"open", KEY_OPEN},
    {"paste", KEY_PASTE},
    {"find", KEY_FIND},
    {"cut", KEY_CUT},
    {"help", KEY_HELP},
    {"menu", KEY_MENU},
    {"calc", KEY_CALC},
    {"setup", KEY_SETUP},
    {"sleep", KEY_SLEEP},
    {"wakeup", KEY_WAKEUP},
    {"file", KEY_FILE},
    {"sendfile", KEY_SENDFILE},
    {"deletefile", KEY_DELETEFILE},
    {"xfer", KEY_XFER},
    {"prog1", KEY_PROG1},
    {"prog2", KEY_PROG2},
    {"www", KEY_WWW},
    {"msdos", KEY_MSDOS},
    {"coffee", KEY_COFFEE},
    {"screenlock", KEY_SCREENLOCK},
    {"direction", KEY_DIRECTION},
    {"cyclewindows", KEY_CYCLEWINDOWS},
    {"mail", KEY_MAIL},
    {"bookmarks", KEY_BOOKMARKS},
    {"computer", KEY_COMPUTER},
    {"back", KEY_BACK},
    {"forward", KEY_FORWARD},
    {"closecd", KEY_CLOSECD},
    {"ejectcd", KEY_EJECTCD},
    {"ejectclosecd", KEY_EJECTCLOSECD},
    {"nextsong", KEY_NEXTSONG},
    {"playpause", KEY_PLAYPAUSE},
    {"previoussong", KEY_PREVIOUSSONG},
    {"stopcd", KEY_STOPCD},
    {"record", KEY_RECORD},
    {"rewind", KEY_REWIND},
    {"phone", KEY_PHONE},
    {"iso", KEY_ISO},
    {"config", KEY_CONFIG},
    {"homepage", KEY_HOMEPAGE},
    {"refresh", KEY_REFRESH},
    {"exit", KEY_EXIT},
    {"move", KEY_MOVE},
    {"edit", KEY_EDIT},
    {"scrollup", KEY_SCROLLUP},
    {"scrolldown", KEY_SCROLLDOWN},
    {"kpleftparen", KEY_KPLEFTPAREN},
    {"kprightparen", KEY_KPRIGHTPAREN},
    {"new", KEY_NEW},
    {"redo", KEY_REDO},
    {"f13", KEY_F13},
    {"f14", KEY_F14},
    {"f15", KEY_F15},
    {"f16", KEY_F16},
    {"f17", KEY_F17},
    {"f18", KEY_F18},
    {"f19", KEY_F19},
    {"f20", KEY_F20},
    {"f21", KEY_F21},
    {"f22", KEY_F22},
    {"f23", KEY_F23},
    {"f24", KEY_F24},
    {"playcd", KEY_PLAYCD},
    {"pausecd", KEY_PAUSECD},
    {"prog3", KEY_PROG3},
    {"prog4", KEY_PROG4},
    {"dashboard", KEY_DASHBOARD},
    {"suspend", KEY_SUSPEND},
    {"close", KEY_CLOSE},
    {"play", KEY_PLAY},
    {"fastforward", KEY_FASTFORWARD},
    {"bassboost", KEY_BASSBOOST},
    {"print", KEY_PRINT},
    {"hp", KEY_HP},
    {"camera", KEY_CAMERA},
    {"sound", KEY_SOUND},
    {"question", KEY_QUESTION},
    {"email", KEY_EMAIL},
    {"chat", KEY_CHAT},
    {"search", KEY_SEARCH},
    {"connect", KEY_CONNECT},
    {"finance", KEY_FINANCE},
    {"sport", KEY_SPORT},
    {"shop", KEY_SHOP},
    {"alterase", KEY_ALTERASE},
    {"cancel", KEY_CANCEL},
    {"brightnessdown", KEY_BRIGHTNESSDOWN},
    {"brightnessup", KEY_BRIGHTNESSUP}};

#define int_to_str(x, y) QString::number(x, y)
#define float_to_str(x) QString::number(x)
